with System;
with Apex_System_Constants;
with Apex_Stack_Trace_Foundation;
with Unchecked_Conversion;
with Apex_Modular_Type_To_String;
with Apex_Address_To_String;

package body Apex_Stack_Trace_Io is

    pragma Suppress_All;

    type Register is mod System.Max_Binary_Modulus;

    package Os_Dependent is

        function Open (Filename : String; Append : Boolean := False)
                      return File_Descriptor;
        pragma Suppress (Elaboration_Check, On => Open);

        procedure Put_String (Fd : File_Descriptor; S : String);
        pragma Suppress (Elaboration_Check, On => Put_String);

        procedure Close (Fd : File_Descriptor);
        pragma Suppress (Elaboration_Check, On => Close);

    end Os_Dependent;

    package body Os_Dependent is separate;


    -- A machine inDependent routine for printing register dumps... up to four
    -- registers per line.
    procedure Dump_Four_Registers (Fd : File_Descriptor;
                                   Name1 : String;
                                   Val1 : Register := 0;
                                   Name2 : String := "";
                                   Val2 : Register := 0;
                                   Name3 : String := "";
                                   Val3 : Register := 0;
                                   Name4 : String := "";
                                   Val4 : Register := 0) is

        function Register_To_String (Reg : Register) return String is
        begin
            return Apex_Modular_Type_To_String.Convert
                      (Apex_Modular_Type_To_String.Largest_Modular_Type (Reg),
                       Register'Size);
        end Register_To_String;

    begin
        Os_Dependent.Put_String (Fd, Name1);
        Os_Dependent.Put_String (Fd, ":");
        Os_Dependent.Put_String (Fd, Register_To_String (Val1));
        if Name2 /= "" then
            Os_Dependent.Put_String (Fd, "  ");
            Os_Dependent.Put_String (Fd, Name2);
            Os_Dependent.Put_String (Fd, ":");
            Os_Dependent.Put_String (Fd, Register_To_String (Val2));
            if Name3 /= "" then
                Os_Dependent.Put_String (Fd, "  ");
                Os_Dependent.Put_String (Fd, Name3);
                Os_Dependent.Put_String (Fd, ":");
                Os_Dependent.Put_String (Fd, Register_To_String (Val3));
                Os_Dependent.Put_String (Fd, "  ");
                if Name4 /= "" then
                    Os_Dependent.Put_String (Fd, Name4);
                    Os_Dependent.Put_String (Fd, ":");
                    Os_Dependent.Put_String (Fd, Register_To_String (Val4));
                end if;
            end if;
        end if;
        Os_Dependent.Put_String (Fd, (1 => Ascii.Lf));
    end Dump_Four_Registers;


    function Open (Filename : String; Append : Boolean := False)
                  return File_Descriptor is
    begin
        return Os_Dependent.Open (Filename, Append);
    end Open;

    procedure Close (Fd : File_Descriptor) is
    begin
        Os_Dependent.Close (Fd);
    end Close;

    package Machine_Dependent is

        procedure Dump_Registers (Fd : File_Descriptor;
                                  Occurrence : Apex_Occurrence_Types.
                                                  Exception_Occurrence_Access);
        pragma Suppress (Elaboration_Check, On => Dump_Registers);

    end Machine_Dependent;

    package body Machine_Dependent is separate;

    procedure Dump_Trace_Label (Fd : File_Descriptor; Label : String) is
    begin
        Os_Dependent.Put_String (Fd, "TL:");
        Os_Dependent.Put_String (Fd, Label);
        Os_Dependent.Put_String (Fd, (1 => Ascii.Lf));
    end Dump_Trace_Label;

    procedure Dump_Registers (Fd : File_Descriptor;
                              Occurrence : Apex_Occurrence_Types.
                                              Exception_Occurrence_Access) is
    begin
        Machine_Dependent.Dump_Registers (Fd, Occurrence);
    end Dump_Registers;

    procedure Dump_Stack_Trace
                 (Fd : File_Descriptor;
                  Context : Apex_Stack_Trace_Types.Frame_Access) is
        Complete : Boolean := False;
    begin
        while not Complete loop
            Os_Dependent.Put_String (Fd, "PC:");
            Os_Dependent.Put_String
               (Fd, Apex_Address_To_String
                       (Apex_Stack_Trace_Foundation.Program_Counter
                           (Context)));
            Os_Dependent.Put_String (Fd, (1 => Ascii.Lf));
            Apex_Stack_Trace_Foundation.Previous_Frame (Context, Complete);
        end loop;
    end Dump_Stack_Trace;

end Apex_Stack_Trace_Io;

